import junit.framework.*;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringBufferInputStream;
import java.util.Vector;

public class AssociationListTest extends TestCase
{
    private AssociationList list;
    private Association assn1;

    private String file1;

    protected void setUp()
    {
	list = new AssociationList();
	assn1 = new Association("key", "value");

        file1 = "[Wallingford]:[Computer Science]\n" +
                "[Prophet]:[Mathematics]";
    }

    public void testContainsNothing()
    {
	assertEquals(false, list.contains(assn1));
    }

    public void testAdd()
    {
	list.add(assn1);
	assertEquals(true, list.contains(assn1));
    }

    public void testAddBrandNewAssociation()
    {
	list.add("key", "value");
	assertEquals(true, list.containsKey("key"));
    }

    public void testContainsAnElement()
    {
	list.add(assn1);
	assertEquals(true, list.contains(assn1));
    }

    public void testContainsAnElementEqualToButNotInTheList()
    {
	Association assn2 = new Association("key", "value");
	list.add(assn1);
	assertEquals(true, list.contains(assn2));
    }

    public void testContainsKey()
    {
	list.add(assn1);
	assertEquals(true, list.containsKey("key"));
    }

    public void testLookup()
    {
	Association assn2 = new Association("key2", "value2");
	Association assn3 = new Association("key2", "value2");
	list.add(assn1);
	list.add(assn2);
	list.add(assn3);
	Vector returnedVector = list.lookup("key2");
	assertEquals(2, returnedVector.size());
    }

    public void testFindAll()
    {
	Association assn2 = new Association("kay2", "value2");
	Association assn3 = new Association("key2", "valum2");
	list.add(assn1);
	list.add(assn2);
	list.add(assn3);
	Vector returnedVector = list.findAll("e");
	assertEquals(3, returnedVector.size());
    }

    public void testRemove()
    {
	Association assn2 = new Association("key2", "value2");
	Association assn3 = new Association("key2", "value2");
	list.add(assn1);
	list.add(assn2);
	list.add(assn3);
	list.remove(assn2);
	assertEquals(true, list.contains(assn1));
	assertEquals(false, list.contains(assn2));
	assertEquals(false, list.contains(assn3));
    }

    public void testUpdate()
    {
	Association assn2 = new Association("key2", "value2");
	Association assn3 = new Association("key2", "value2");
	list.add(assn1);
	list.add(assn2);
	list.add(assn3);
	list.update("key2", " updated");
	Vector updatedVector = list.findAll("value2 updated");
	assertEquals(2, updatedVector.size());
    }

    // ------------------------------------------------------------

    public void testSizeOfEmptyListIsZero()
    {
        assertEquals( 0, list.size() );
    }

    public void testSizeOfListAfterTwoAddsIsTwo()
    {
        addTwoAssociations();
        assertEquals( 2, list.size() );
    }

    public void testClearResultsInListOfSizeZero()
    {
        addTwoAssociations();
        list.clear();
        assertEquals( 0, list.size() );
    }

    public void testSaveEmptyListToFileCreatesEmptyFile()
    {
        list.saveToFile( "test01.dat" );
        assertEquals( 0, linesInFile( "test01.dat" ) );
    }

    public void testSaveTwoElementListToFileCreatesTwoElementFile()
    {
        addTwoAssociations();
        list.saveToFile( "test02.dat" );
        assertEquals( 2, linesInFile( "test02.dat" ) );
    }

    // ------------------------------------------------------------

    public void testLoadFromEmptyFileUsingStream()
    {
        list.loadFromFile( new StringBufferInputStream("") );
        assertEquals( 0, list.size() );
    }

    public void testLoadFromTwoElementFileUsingStream()
    {
        list.loadFromFile( new StringBufferInputStream(file1) );
        assertEquals( 2, list.size() );
        assertTrue  ( list.containsKey( "Wallingford" ) );
        assertTrue  ( list.containsKey( "Prophet"     ) );
    }

    public void testLoadFromTwoElementFileTwiceUsingStream()
    {
        list.loadFromFile( new StringBufferInputStream(file1) );
        list.loadFromFile( new StringBufferInputStream(file1) );
        assertEquals( 4, list.size() );
        assertEquals( 2, list.lookup( "Wallingford" ).size() );
        assertEquals( 2, list.lookup( "Prophet"     ).size() );
    }

    // ------------------------------------------------------------

    public void testLoadSortedFromEmptyFileCreatesEmptyList()
    {
        list.loadFromFileSorted( new StringBufferInputStream("") );
        assertEquals( 0, list.size() );
    }

    public void testLoadSortedFromTwoElementFileCreatesTwoElementList()
    {
        list.loadFromFileSorted( new StringBufferInputStream(file1) );
        assertEquals( 2, list.size() );
        assertTrue  ( list.containsKey( "Wallingford" ) );
        assertTrue  ( list.containsKey( "Prophet"     ) );
        // How do we know they are in order?
        list.saveToFile( "test03.dat" );
    }

    public void testLoadSortedFromTwoElementFileAddsSortedElementsAtEnd()
    {
        list.loadFromFile      ( new StringBufferInputStream(file1) );
        list.loadFromFileSorted( new StringBufferInputStream(file1) );
        assertEquals( 4, list.size() );
        assertEquals( 2, list.lookup( "Wallingford" ).size() );
        assertEquals( 2, list.lookup( "Prophet"     ).size() );
        // How do we know they are in order?
        list.saveToFile( "test04.dat" );
    }

    // ------------------------------------------------------------

    private void addTwoAssociations()
    {
        list.add( new Association( "Wallingford", "Computer Science" ) );
        list.add( new Association( "Prophet"    , "Mathematics"      ) );
    }

    private int linesInFile( String filename )
    {
        try {
            BufferedReader inputFile = new BufferedReader(
                                           new FileReader( filename ) );
            String buffer = null;
            int    count  = 0;
            while ( true )
            {
                buffer = inputFile.readLine();
                if ( buffer == null ) break;
                count++;
            }

            inputFile.close();
            return count;
        } catch ( IOException e ) {
            System.out.println( e );
        }

        return -1;
    }

    // ------------------------------------------------------------

    public static Test suite()
    {
        return new TestSuite( AssociationListTest.class );
    }

    public static void main( String[] args )
    {
        junit.textui.TestRunner.run( suite() );
    }
}
